# xDevs.com Python 10V test for 3458A
# http://xdevs.com/guide/ni_gpib_rpi/
# G9330 10Kohm transfer test
#  3458A calibrated GPIB 3
#  K2002 for RTD GPIB 9
#  BME280 sensor I2C
# 
import os.path
import sys
import Gpib
import time
import ftplib
import numbers
import signal
from Adafruit_BME280 import *

root_dir = '/repo/3458/'
fileName4 = root_dir + 'sr104_10k_3458_nplc100_mm.csv'

cnt = 0
tread = 20
reflevel = 100
refhp = 100
k10v = 10000.050
tec_start = 24
tec_stop = 50
tec_step = 0.1
tec_time = 50

tec_rtd = 0.3
temp = 18.0
res_rtd = 0.3
exttemp = 25.0
rh = 50.0
pascals = 100000.00
hectopascals = 1000.00
tset = 18.0

class Timeout():
  """Timeout class using ALARM signal"""
  class Timeout(Exception): pass

  def __init__(self, sec):
    self.sec = sec

  def __enter__(self):
    signal.signal(signal.SIGALRM, self.raise_timeout)
    signal.alarm(self.sec)

  def __exit__(self, *args):
    signal.alarm(0) # disable alarm

  def raise_timeout(self, *args):
    raise Timeout.Timeout()

class scpi_meter():
    temp = 38.5
    data = ""
    ppm = 0.0
    status_flag = 1
    temp_status_flag = 1
    global exttemp
    global rh
    global hectopascals
    global res_rtd

    def __init__(self,gpib,reflevel,name):
        self.gpib = gpib
        self.inst = Gpib.Gpib(0,self.gpib, timeout=90) # SCPI GPIB Address = self.gpib
        self.reflevel = reflevel
        self.name = name
        #self.init_inst()

    def init_inst(self):
        # Setup SCPI DMM
        self.inst.clear()
        self.inst.write("*RST")
        self.inst.write("*CLR")
        self.inst.write(":SYST:AZER:TYPE SYNC")
        self.inst.write(":SYST:LSYN:STAT ON")
        #self.inst.write(":sens:temp:tran rtd")      #select thermistor
        #self.inst.write(":sens:temp:rtd:type user")    #RTD from G9330
        #self.inst.write(":sens:temp:rtd:alph 0.00375") #RTD from G9330
        #self.inst.write(":sens:temp:rtd:beta 0.160")   #RTD from G9330
        #self.inst.write(":sens:temp:rtd:delt 1.605")   #RTD from G9330
        #self.inst.write(":sens:temp:rtd:rzer 1000")    #RTD from G9330
        #self.inst.write(":SENS:FUNC 'TEMP'")
        #self.inst.write(":SENS:TEMP:DIG 7")
        #self.inst.write(":SENS:TEMP:NPLC 10")
        self.inst.write(":SENS:FUNC 'FRES'")
        self.inst.write(":SENS:FRES:AVER:STAT OFF")
        self.inst.write(":SENS:FRES:RANG 20E3")
        self.inst.write(":SENS:FRES:DIG 8")
        self.inst.write(":SENS:FRES:NPLC 10")
        self.inst.write(":FORM:ELEM READ")

    def read_data(self,cmd):
        data_float = 0.0
        data_str = ""
        self.inst.write(cmd)
        try:
            with Timeout(90):
                data_str = self.inst.read()
        except Timeout.Timeout:
            print ("Timeout exception from dmm %s on read_data() inst.read()\n" % self.name)
            return (0,float(0))
        #print ("Reading from dmm %s = %s" % (self.name,data_str))
        try:
            data_float = float(data_str)
        except ValueError:
            print("Exception thrown by dmm %s on read_data() - ValueError = %s\n" % (self.name,data_str))
            return (0,float(0)) # Exception on float conversion, 0 = error
        return (1,data_float) # Good read, 1 = converted to float w/o exception

    def get_data(self):
	global tec_rtd
        self.status_flag,data = self.read_data("READ?")
        if (self.status_flag):
            self.data = data
            self.ppm = ((float(self.data) / self.reflevel)-1)*1E6
	    self.res_rtd = data
        tec_rtd = float(self.data)
        return self.data

    def get_data_fres(self):
	global k10v
        self.status_flag,data = self.read_data("READ?")
        if (self.status_flag):
            self.data = data
            self.ppm = ((float(self.data) / self.reflevel)-1)*1E6
	    self.res_rtd = data
        k10v = float(self.data)
        return self.data

    def get_data_status(self):
        return self.status_flag

    def write_data(self, fileHandle):
	res_rtd = self.data
	tempc = float(res_rtd)
        fileHandle.write(";%2.3f;\r\n" % tempc);
        #print time.strftime("%d/%m/%Y-%H:%M:%S;") + ("\033[1;31m[%8d]: %2.8f , dev %4.2f ppm,\033[1;39m  EXT_T:%3.2f , RH:%3.2f , Press:%4.2f hPa" % (cnt, float(self.data),float(self.ppm),float(exttemp),float(rh),float(hectopascals) ) )
        #fileHandle.write (time.strftime("%d/%m/%Y-%H:%M:%S;") + ("%16.8f;%16.8f;%3.1f;%3.2f;%3.2f;%4.2f;\r\n" % (float(self.data),float(self.reflevel),float(self.ppm),float(exttemp),float(rh),float(hectopascals) ) ))

    def print_ppm(self):
	res_rtd = self.data
        tempc = float(res_rtd)
        print ("%2.3f" % tempc)
	self.inst.write(":DISP:WIND2:TEXT:DATA \"%2.3f    \"" % tempc)
	#self.inst.write(":DISP:WIND2:TEXT:DATA \"  \"")


class meter():
    temp = 24.0
    data = ""
    ppm = 0.0
    status_flag = 1
    temp_status_flag = 1
    global exttemp
    global rh
    global hectopascals
    global res_rtd
    global tec_rtd

    def __init__(self,gpib,refhp,name):
        self.gpib = gpib
        self.inst = Gpib.Gpib(0,self.gpib, timeout=160) # 3458A GPIB Address = self.gpib
        self.refhp = refhp
        self.name = name
        self.init_inst()

    def init_inst(self):
        # Setup HP 3458A
        self.inst.clear()
        self.inst.write("PRESET NORM")
        self.inst.write("OFORMAT ASCII")
        self.inst.write("OHMF 10000")
        self.inst.write("TARM HOLD")
        self.inst.write("TRIG AUTO")
        self.inst.write("NPLC 100")
        self.inst.write("AZERO ON")
        self.inst.write("OCOMP ON")
        self.inst.write("LFILTER ON")
        self.inst.write("NRDGS 1,AUTO")
        self.inst.write("MEM OFF")
        self.inst.write("END ALWAYS")
        self.inst.write("NDIG 9")
        self.inst.write("DELAY 3") # 2 second delay to mitigate OCOMP accuracy issue due DA

    def read_data(self,cmd):
        data_float = 0.0
        data_str = ""
        self.inst.write(cmd)
    	time.sleep(0.1)
        try:
            with Timeout(160):
		time.sleep(0.1)
                data_str = self.inst.read()
        except Timeout.Timeout:
            print ("Timeout exception from dmm %s on read_data() inst.read()\n" % self.name)
            return (0,float(0))
        #print ("Reading from dmm %s = %s" % (self.name,data_str))
        try:
            data_float = float(data_str)
        except ValueError:
            print("Exception thrown by dmm %s on read_data() - ValueError = %s\n" % (self.name,data_str))
            return (0,float(0)) # Exception on float conversion, 0 = error
        return (1,data_float) # Good read, 1 = converted to float w/o exception

    def get_temp(self):
        self.inst.write("TARM SGL,1")
	time.sleep(0.1)        
	self.temp_status_flag,temp = self.read_data("TEMP?")
	time.sleep(0.1)
        if (self.temp_status_flag):
            self.temp = temp
        return self.temp

    def acalall(self):
	self.inst.write("ACAL ALL")
	time.sleep(250)
	return 0

    def get_temp_status(self):
        return self.temp_status_flag

    def get_data(self):
        self.status_flag,data = self.read_data("TARM SGL,1")
	time.sleep(0.1)
        if (self.status_flag):
            self.data = (data)
            self.ppm = ((float(self.data) / self.refhp)-1)*1E6
        return self.data

    def get_data_status(self):
        return self.status_flag

    def write_data(self,fileHandle):
	global tec_rtd
	global k10v
        print time.strftime("%d/%m/%Y-%H:%M:%S;") + ("\033[1;36m[%8d]: %2.9f , dev %4.4f ppm, T:%3.1f\033[1;39m , EXT_T:%3.2f , RH:%3.2f , Press:%4.2f hPa, RTD %f, %8.7f" % (cnt, float(self.data),float(self.ppm),float(self.temp),float(exttemp),float(rh),float(hectopascals),tec_rtd,k10v ) )
        fileHandle.write (time.strftime("%d/%m/%Y-%H:%M:%S;") + ("%16.8f;%16.8f;%3.1f;%4.3f;%3.2f;%3.2f;%4.2f;%3.2f;%4.2f;%3.3f;%8.7f;\r\n" % (float(self.data),float(self.refhp),float(self.temp),float(self.ppm),float(exttemp),float(rh),float(hectopascals),0.0,23.4,tec_rtd,k10v ) ))

    def print_ppm(self):
        self.inst.write("DISP OFF, \"%3.3f ppm\"" % float(self.ppm))
        time.sleep(0.1)
        #self.inst.write("DISP OFF, \" \"")
        
# Read Temperature, Humidity, and Barometric Pressure from BME280
def get_THP():
    global exttemp
    global rh
    global pascals
    global hectopascals
    exttemp = sensor.read_temperature()
    rh = sensor.read_humidity()
    pascals = sensor.read_pressure()
    hectopascals = pascals / 100
    time.sleep(0.1)

# Check if file exists, if not create it and add header
def create_local_file(fileName):
    if (os.path.isfile(fileName) == False):
        with open(fileName, 'a') as o:
            o.write("date;hp3458a;level;temp;ppm_level;ext_temp;rh;pressure;tec_ttd;res_rtd;tec_rtd;k10v;\r\n")
            print ("file %s does not exist\r\n" % fileName) 
    else: 
        print ("file %s exists\r\n" % fileName)

meter1 = meter(3,refhp,"3458A_00000") # Gpib 3
#meter2510 = tec_meter(25,reflevel,"2510_00000") # Gpib 25
meter2002 = scpi_meter(9,reflevel,"2002_00000") # Gpib 10
meter2002.init_inst()

meter1.get_temp()

#Setup temp/humidity/pressure sensor BME280
# OSAMPLE = oversampling mode
sensor = BME280(mode=BME280_OSAMPLE_8)
get_THP() # Read Temp, RH, Pressure from sensor

# Create file for 1st 3458A and write header if needed
create_local_file(fileName4)

temp = meter1.get_temp()
prevt = meter1.get_temp()

meter1.print_ppm()

while cnt <= 100000:
    cnt+=1

    with open(fileName4, 'a') as o1: # Open file handles for both files
        get_THP() # Read Temp, RH, Press from sensor
        tread = tread - 1
        if (tread == 0):
            tread = 150
            temp = meter1.get_temp()
	    if (temp > prevt + 0.2) or (temp < prevt - 0.2):
		meter1.acalall()
		prevt = temp

	meter2002.get_data_fres()
        meter1.get_data()
	if (meter1.get_data_status()):
    	    meter1.write_data(o1)
    
        o1.close()
